@@ -65,25 +65,19 @@ static const char* get_event_name(rb_event_flag_t event)
         return "fiber-switch";
     case RUBY_EVENT_RAISE:
         return "raise";
+    case RUBY_INTERNAL_EVENT_NEWOBJ:
+        return "newobj";
     default:
         return "unknown";
     }
 }
 
-VALUE get_fiber(prof_profile_t* profile)
-{
-    if (profile->merge_fibers)
-        return rb_thread_current();
-    else
-        return rb_fiber_current();
-}
-
 thread_data_t* check_fiber(prof_profile_t* profile, double measurement)
 {
     thread_data_t* result = NULL;
 
-    /* Get the current thread and fiber information. */
-    VALUE fiber = get_fiber(profile);
+    // Get the current fiber
+    VALUE fiber = rb_fiber_current();
 
     /* We need to switch the profiling context if we either had none before,
      we don't merge fibers and the fiber ids differ, or the thread ids differ. */
@@ -144,11 +138,7 @@ prof_method_t* check_method(VALUE profile, rb_trace_arg_t* trace_arg, rb_event_f
     if (klass == cProfile)
         return NULL;
 
-#ifdef HAVE_RB_TRACEARG_CALLEE_ID
     VALUE msym = rb_tracearg_callee_id(trace_arg);
-#else
-    VALUE msym = rb_tracearg_method_id(trace_arg);
-#endif
 
     st_data_t key = method_key(klass, msym);
 
@@ -164,7 +154,7 @@ prof_method_t* check_method(VALUE profile, rb_trace_arg_t* trace_arg, rb_event_f
         int source_line = (event != RUBY_EVENT_C_CALL ? FIX2INT(rb_tracearg_lineno(trace_arg)) : 0);
         result = create_method(profile, key, klass, msym, source_file, source_line);
     }
-    
+
     return result;
 }
 
@@ -172,7 +162,7 @@ prof_method_t* check_method(VALUE profile, rb_trace_arg_t* trace_arg, rb_event_f
 static void prof_trace(prof_profile_t* profile, rb_trace_arg_t* trace_arg, double measurement)
 {
     static VALUE last_fiber = Qnil;
-    VALUE fiber = get_fiber(profile);
+    VALUE fiber = rb_fiber_current();
 
     rb_event_flag_t event = rb_tracearg_event_flag(trace_arg);
     const char* event_name = get_event_name(event);
@@ -180,11 +170,7 @@ static void prof_trace(prof_profile_t* profile, rb_trace_arg_t* trace_arg, doubl
     VALUE source_file = rb_tracearg_path(trace_arg);
     int source_line = FIX2INT(rb_tracearg_lineno(trace_arg));
 
-#ifdef HAVE_RB_TRACEARG_CALLEE_ID
     VALUE msym = rb_tracearg_callee_id(trace_arg);
-#else
-    VALUE msym = rb_tracearg_method_id(trace_arg);
-#endif
 
     unsigned int klass_flags;
     VALUE klass = rb_tracearg_defined_class(trace_arg);
@@ -259,13 +245,13 @@ static void prof_event_hook(VALUE trace_point, void* data)
                 {
                     frame = prof_frame_push(thread_data->stack, call_tree, measurement, RTEST(profile_t->paused));
                 }
-                
+
                 thread_data->call_tree = call_tree;
             }
-            
+
             frame->source_file = rb_tracearg_path(trace_arg);
             frame->source_line = FIX2INT(rb_tracearg_lineno(trace_arg));
-            
+
             break;
         }
         case RUBY_EVENT_CALL:
@@ -288,7 +274,7 @@ static void prof_event_hook(VALUE trace_point, void* data)
             }
             else if (!frame && thread_data->call_tree)
             {
-                // There is no current parent - likely we have returned out of the highest level method we have profiled so far. 
+                // There is no current parent - likely we have returned out of the highest level method we have profiled so far.
                 // This can happen with enumerators (see fiber_test.rb). So create a new dummy parent.
                 prof_method_t* parent_method = check_parent_method(profile, thread_data);
                 parent_call_tree = prof_call_tree_create(parent_method, NULL, Qnil, 0);
@@ -427,7 +413,7 @@ static void prof_profile_mark(void* data)
         rb_st_foreach(profile->exclude_methods_tbl, prof_profile_mark_methods, 0);
 }
 
-/* Freeing the profile creates a cascade of freeing. It frees its threads table, which frees 
+/* Freeing the profile creates a cascade of freeing. It frees its threads table, which frees
    each thread and its associated call treee and methods. */
 static void prof_profile_ruby_gc_free(void* data)
 {
@@ -487,7 +473,6 @@ static VALUE prof_allocate(VALUE klass)
     profile->include_threads_tbl = NULL;
     profile->running = Qfalse;
     profile->allow_exceptions = false;
-    profile->merge_fibers = false;
     profile->exclude_methods_tbl = method_table_create();
     profile->running = Qfalse;
     profile->tracepoints = rb_ary_new();
@@ -529,9 +514,6 @@ prof_stop_threads(prof_profile_t* profile)
                       If not specified, defaults to RubyProf::WALL_TIME.
    allow_exceptions:  Whether to raise exceptions encountered during profiling,
                       or to suppress all exceptions during profiling
-   merge_fibers:      Whether profiling data for a given thread's fibers should all be
-                      subsumed under a single entry. Basically only useful to produce
-                      callgrind profiles.
    track_allocations: Whether to track object allocations while profiling. True or false.
    exclude_common:    Exclude common methods from the profile. True or false.
    exclude_threads:   Threads to exclude from the profiling results.
@@ -546,7 +528,6 @@ static VALUE prof_initialize(int argc, VALUE* argv, VALUE self)
     VALUE include_threads = Qnil;
     VALUE exclude_common = Qnil;
     VALUE allow_exceptions = Qfalse;
-    VALUE merge_fibers = Qfalse;
     VALUE track_allocations = Qfalse;
 
     int i;
@@ -566,7 +547,6 @@ static VALUE prof_initialize(int argc, VALUE* argv, VALUE self)
             mode = rb_hash_aref(mode_or_options, ID2SYM(rb_intern("measure_mode")));
             track_allocations = rb_hash_aref(mode_or_options, ID2SYM(rb_intern("track_allocations")));
             allow_exceptions = rb_hash_aref(mode_or_options, ID2SYM(rb_intern("allow_exceptions")));
-            merge_fibers = rb_hash_aref(mode_or_options, ID2SYM(rb_intern("merge_fibers")));
             exclude_common = rb_hash_aref(mode_or_options, ID2SYM(rb_intern("exclude_common")));
             exclude_threads = rb_hash_aref(mode_or_options, ID2SYM(rb_intern("exclude_threads")));
             include_threads = rb_hash_aref(mode_or_options, ID2SYM(rb_intern("include_threads")));
@@ -587,7 +567,6 @@ static VALUE prof_initialize(int argc, VALUE* argv, VALUE self)
     }
     profile->measurer = prof_get_measurer(NUM2INT(mode), track_allocations == Qtrue);
     profile->allow_exceptions = (allow_exceptions == Qtrue);
-    profile->merge_fibers = (merge_fibers == Qtrue);
 
     if (exclude_threads != Qnil)
     {
@@ -679,7 +658,7 @@ static VALUE prof_start(VALUE self)
 
     profile->running = Qtrue;
     profile->paused = Qfalse;
-    profile->last_thread_data = threads_table_insert(profile, get_fiber(profile));
+    profile->last_thread_data = threads_table_insert(profile, rb_fiber_current());
 
     /* open trace file if environment wants it */
     trace_file_name = getenv("RUBY_PROF_TRACE");
@@ -769,11 +748,7 @@ static VALUE prof_stop(VALUE self)
     {
         if (trace_file != stderr && trace_file != stdout)
         {
-#ifdef _MSC_VER
-            _fcloseall();
-#else
             fclose(trace_file);
-#endif
         }
         trace_file = NULL;
     }
